# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-17 23:07
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('google_address', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AddressComponent',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('long_name', models.CharField(max_length=400)),
                ('short_name', models.CharField(max_length=400)),
            ],
        ),
        migrations.CreateModel(
            name='AddressComponentType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='GoogleAddress',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('typed_address', models.CharField(blank=True, max_length=400, null=True)),
                ('typed_address2', models.CharField(blank=True, max_length=400, null=True)),
                ('address_line', models.CharField(blank=True, max_length=400, null=True)),
                ('city_state', models.CharField(blank=True, max_length=400, null=True)),
                ('lat', models.FloatField(blank=True, null=True, verbose_name='lat')),
                ('lng', models.FloatField(blank=True, null=True, verbose_name='lng')),
                ('address_components', models.ManyToManyField(to='google_address.AddressComponent')),
            ],
        ),
        migrations.CreateModel(
            name='GoogleRegion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('region_name', models.CharField(max_length=400)),
                ('filter_by', models.CharField(max_length=400)),
            ],
        ),
        migrations.AddField(
            model_name='addresscomponent',
            name='types',
            field=models.ManyToManyField(to='google_address.AddressComponentType'),
        ),
    ]



from django.contrib import admin
from django import forms
from django.utils.translation import ugettext_lazy as _

from google_address.models import Address


class GoogleAddressAdmin(admin.ModelAdmin):
  fields = [
    'id', 'raw', 'raw2'
  ]

  list_display = [
    'id', 'raw', 'raw2'
  ]

  list_filter = []

  list_editable = []

  search_fields = ['raw', 'raw2', 'address_line']

  readonly_fields = [
    'id'
  ]

  raw_id_fields = []


admin.site.register(Address, GoogleAddressAdmin)





from google_address import helpers
import requests

class GoogleAddressApi():
  url = 'https://maps.googleapis.com/maps/api/geocode/json?address={address}'
  key = None

  def __init__(self):
    # Set key
    self.key = helpers.get_settings().get("API_KEY", None)

    # Set language
    self.language = helpers.get_settings().get("API_LANGUAGE", "en_US")

  def _get_url(self):
    url = self.url

    if self.key:
      url = "{}&key={}".format(url, self.key)

    if self.language:
      url = "{}&language={}".format(url, self.language)

    return url

  def query(self, raw):
    url = self._get_url().format(address=raw)

    r = requests.get(url)
    data = r.json()
    return data



from django.test import TestCase
from django.test.utils import override_settings

from google_address.models import Address
from google_address.models import AddressComponent
from google_address.models import AddressComponentType

def remove_component(address, types):
  for component in address.address_components.all():
    for type in component.types.all():
      if type.name in types:
        component.delete()

  return address


@override_settings(GOOGLE_ADDRESS={'API_LANGUAGE': 'en_US'})
class AddressModelTestCase(TestCase):
  def test_api_call(self):
    """Assert Address calls google API and get address"""
    a = Address(raw="Rua Teçaindá, 81, SP", raw2="Casa")
    a.save()

    a = Address.objects.get(pk=a.pk)
    self.assertTrue(a.raw == "Rua Teçaindá, 81, SP")
    self.assertTrue(a.raw2 == "Casa")
    self.assertTrue(a.address_line == "Rua Teçaindá, 81, Pinheiros, São Paulo, SP, Brazil")
    self.assertTrue(a.__str__() == "Rua Teçaindá, 81, Pinheiros, São Paulo, SP, Brazil")
    self.assertTrue(a.lat)
    self.assertTrue(a.lng)

    a.raw="Rua Capote Valente, 701, SP"
    a.save()
    a = Address.objects.get(pk=a.pk)
    self.assertTrue(a.raw == "Rua Capote Valente, 701, SP")
    self.assertTrue(a.raw2 == "Casa")
    self.assertTrue(a.address_line == "Rua Capote Valente, 701, Pinheiros, São Paulo, SP, Brazil")
    self.assertTrue(a.__str__()  == "Rua Capote Valente, 701, Pinheiros, São Paulo, SP, Brazil")
    self.assertTrue(a.lat)
    self.assertTrue(a.lng)

    a.address_line=None
    self.assertTrue(a.__str__() == "")

  def test_locality(self):
    """Assert AddressModel.get_city_state preference order is locality, administrative_area_2"""
    a = Address(raw="Chicago")
    a.save()
    self.assertTrue("Chicago" in a.get_city_state())

    a = remove_component(a, ['locality'])
    self.assertTrue("Cook" in a.get_city_state())

  def test_no_country(self):
    """Assert AddressModel.get_country_code returns None instead of rasing AttributeError if no country exists"""
    a = Address(raw="Chicago")
    a.save()

    AddressComponent.objects.all().delete()

    self.assertTrue(a.get_country_code() == None)

  def test_signal_returns_in_case_of_no_result(self):
    """Assert AddressModel.get_country_code returns None instead of rasing AttributeError if no country exists"""
    a = Address(raw="abcdefghijklmnopqrstuvwxyz")
    a.save()
    self.assertTrue(a.address_line == None)


class AddressComponentTypeModelTestCase(TestCase):
  def test_str_call(self):
    """Assert AddressComponentType __str__ returns name"""
    a = AddressComponentType(name="xyz")
    a.save()

    self.assertTrue(a.__str__() == "xyz")

class AddressComponentModelTestCase(TestCase):
  def test_str_call(self):
    """Assert AddressComponent __str__ returns long name"""
    a = AddressComponent(short_name="short", long_name="long")
    a.save()

    self.assertTrue(a.__str__() == "long")



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-18 01:33
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('google_address', '0003_auto_20170417_2356'),
    ]

    operations = [
        migrations.RenameField(
            model_name='googleaddress',
            old_name='typed_address2',
            new_name='raw2',
        ),
    ]



from django.test import TestCase
from django.test.utils import override_settings

from google_address.api import GoogleAddressApi

class GoogleAddressApiTestCase(TestCase):
  def test_can_request_without_key(self):
    """ Assert it's possible to make requests without setting a key """
    url = GoogleAddressApi()._get_url()
    self.assertTrue("key=" not in url)

  @override_settings(GOOGLE_ADDRESS={'API_KEY': 'test'})
  def test_can_request_with_key(self):
    """ Assert it's possible to make requests setting a key """
    url = GoogleAddressApi()._get_url()
    self.assertTrue("key=test" in url)




from django.db import models
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q, Count
from django.db.models.signals import post_save
from django.dispatch import receiver

from google_address import helpers
from google_address.api import GoogleAddressApi

class AddressComponentType(models.Model):
  name = models.CharField(max_length=100)

  def __str__(self):
    return self.name

class AddressComponent(models.Model):
  long_name = models.CharField(max_length=400)
  short_name = models.CharField(max_length=400)
  types = models.ManyToManyField(AddressComponentType)

  def __str__(self):
    return self.long_name

  @staticmethod
  def get_or_create_component(api_component):
    # Look for component with same name and type
    component = AddressComponent.objects.annotate(count=Count('types')).filter(long_name=api_component['long_name'], short_name=api_component['short_name'])
    for component_type in api_component['types']:
      component = component.filter(types__name=component_type)
    component = component.filter(count=len(api_component['types']))

    if not component.count():
    # Component not found, creating
      component = AddressComponent(long_name=api_component['long_name'], short_name=api_component['short_name'])
      component.save()
    else:
      # We clear and recreate types because
      # sometimes google changes types for a given component
      component = component.first()
      component.types.clear()
      component.save()

    # Add types for component
    for api_component_type in api_component['types']:
      try:
        component_type = AddressComponentType.objects.get(name=api_component_type)
      except ObjectDoesNotExist:
        component_type = AddressComponentType(name=api_component_type)
        component_type.save()
      component.types.add(component_type)

    return component

class Address(models.Model):
  raw = models.CharField(max_length=400, blank=True, null=True)
  raw2 = models.CharField(max_length=400, blank=True, null=True)
  address_line = models.CharField(max_length=400, blank=True, null=True)
  city_state = models.CharField(max_length=400, blank=True, null=True)
  lat = models.FloatField('lat', blank=True, null=True)
  lng = models.FloatField('lng', blank=True, null=True)
  address_components = models.ManyToManyField(AddressComponent)

  def get_city_state(self):
    state = self.address_components.filter(types__name='administrative_area_level_1')
    county = self.address_components.filter(types__name='administrative_area_level_2')
    locality = self.address_components.filter(types__name='locality')

    s = u""
    if locality.count():
      s += u"{}, ".format(locality[0].long_name)
    elif county.count():
      s += u"{}, ".format(county[0].long_name)

    if state.count():
      s += state[0].short_name

    return s

  def get_address(self):
    # Components types for address
    address = {'route': '', 'sublocality_level_1': '', 'administrative_area_level_2': '', 'administrative_area_level_1': '', 'country': '', 'street_number': ''}

    # Fill address dict
    for component in self.address_components.all():
      for component_type in component.types.all():
        if component_type.name in address:
          address[component_type.name] = {'short_name': component.short_name, 'long_name': component.long_name}

    # Build address string
    string_address = ''
    if 'route' in address and isinstance(address['route'], dict):
      string_address += '{}, '.format(address['route']['long_name'])
    if 'route' in address and isinstance(address['street_number'], dict):
      string_address += '{}, '.format(address['street_number']['long_name'])
    if 'sublocality_level_1' in address and isinstance(address['sublocality_level_1'], dict):
      string_address += '{}, '.format(address['sublocality_level_1']['long_name'])
    if 'administrative_area_level_2' in address and isinstance(address['administrative_area_level_2'], dict):
      string_address += '{}, '.format(address['administrative_area_level_2']['long_name'])
    if 'administrative_area_level_1' in address and isinstance(address['administrative_area_level_1'], dict):
      string_address += '{}, '.format(address['administrative_area_level_1']['short_name'])
    if 'country' in address and isinstance(address['country'], dict):
      string_address += '{}, '.format(address['country']['long_name'])

    string_address = string_address.strip().strip(',')

    return string_address

  def get_country_code(self):
    try:
      return self.address_components.filter(types__name='country').first().short_name.lower()
    except (AttributeError):
      return None

  def __str__(self):
    if self.address_line:
      return self.address_line
    return ""


@receiver(post_save, sender=Address)
def update_address(sender, instance, **kwargs):
  # If raw == True, we should not modify the record
  #
  # https://docs.djangoproject.com/en/1.11/ref/signals/#post-save
  if kwargs.get('raw', False): # pragma: no cover
    return None

  response = GoogleAddressApi().query(instance.raw)

  if len(response["results"]) > 0:
    result = response["results"][0]
  else:
    return False

  instance.address_components.clear()
  for api_component in result['address_components']:
    component = AddressComponent.get_or_create_component(api_component)
    instance.address_components.add(component)

  try:
    if result["geometry"]:
      Address.objects.filter(pk=instance.pk).update(lat=result['geometry']['location']['lat'], lng=result['geometry']['location']['lng'])
  except: #pragma: no cover
    pass

  # Using update to avoid post_save signal
  instance.address_line = instance.get_address()
  Address.objects.filter(pk=instance.pk).update(address_line=instance.address_line, city_state=instance.get_city_state())



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-17 23:56
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('google_address', '0002_auto_20170417_2307'),
    ]

    operations = [
        migrations.RenameField(
            model_name='googleaddress',
            old_name='typed_address',
            new_name='raw',
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
from pip.req import parse_requirements

install_reqs = parse_requirements('requirements.txt', session=False)
reqs = [str(ir.req) for ir in install_reqs]

setup(
    name='django-google-address',
    version='1.0.0',
    author=u'Leonardo Arroyo',
    author_email='contato@leonardoarroyo.com',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/leonardoarroyo/django-google-address',
    download_url = 'https://github.com/leonardoarroyo/django-google-address/tarball/1.0.0',
    license='MIT',
    description='',
    long_description=open('README.rst', encoding='utf-8').read(),
    zip_safe=False,
    install_requires = reqs
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-18 23:04
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('google_address', '0004_auto_20170418_0133'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='GoogleAddress',
            new_name='Address',
        ),
    ]
